@@ -23,6 +23,8 @@ if sys.version_info[0] == 3:
 
     bytebuffer = memoryview
 
+    from collections import OrderedDict
+
 elif sys.version_info[0] == 2:
     import httplib as httpclient
     from urlparse import urlparse
@@ -32,6 +34,10 @@ elif sys.version_info[0] == 2:
 
     bytebuffer = buffer
 
+    if sys.version_info[1] >= 7:
+        from collections import OrderedDict  # For Py 2.7+
+    else:
+        from ordereddict import OrderedDict  # Works only on 2.6
 else:
     raise ImportError("Unknown python version: {0}".format(sys.version_info))
 
