@@ -39,6 +39,7 @@ import azurelinuxagent.common.logger as logger
 import azurelinuxagent.common.utils.fileutil as fileutil
 import azurelinuxagent.common.utils.restutil as restutil
 import azurelinuxagent.common.utils.textutil as textutil
+from azurelinuxagent.common.cgroupconfigurator import CGroupConfigurator
 
 from azurelinuxagent.common.event import add_event, add_periodic, \
                                     elapsed_milliseconds, \
@@ -55,7 +56,7 @@ from azurelinuxagent.common.utils.flexible_version import FlexibleVersion
 from azurelinuxagent.common.version import AGENT_NAME, AGENT_VERSION, AGENT_LONG_VERSION, \
                                             AGENT_DIR_GLOB, AGENT_PKG_GLOB, \
                                             AGENT_PATTERN, AGENT_NAME_PATTERN, AGENT_DIR_PATTERN, \
-                                            CURRENT_AGENT, CURRENT_VERSION, \
+                                            CURRENT_AGENT, CURRENT_VERSION, DISTRO_NAME, DISTRO_VERSION, \
                                             is_current_agent_installed
 
 from azurelinuxagent.ga.exthandlers import HandlerManifest
@@ -165,14 +166,10 @@ class UpdateHandler(object):
 
             logger.verbose(u"Agent {0} launched with command '{1}'", agent_name, agent_cmd)
 
-            # If the most current agent is the installed agent and update is enabled,
-            # assume updates are likely available and poll every second.
-            # This reduces the start-up impact of finding / launching agent updates on
-            # fresh VMs.
-            if latest_agent is None and conf.get_autoupdate_enabled():
-                poll_interval = 1
-            else:
-                poll_interval = CHILD_POLL_INTERVAL
+            # Setting the poll interval to poll every second to reduce the agent provisioning time;
+            # The daemon shouldn't wait for 60secs before starting the ext-handler in case the
+            # ext-handler kills itself during agent-update during the first 15 mins (CHILD_HEALTH_INTERVAL)
+            poll_interval = 1
 
             ret = None
             start_time = time.time()
@@ -245,20 +242,33 @@ class UpdateHandler(object):
         self.child_process = None
         return
 
-    def run(self):
+    def run(self, debug=False):
         """
         This is the main loop which watches for agent and extension updates.
         """
 
         try:
+            # NOTE: Do not add any telemetry events until after the monitoring handler has been started with the
+            # call to 'monitor_thread.run()'. That method call initializes the protocol, which is needed in order to
+            # load the goal state and update the container id in memory. Any telemetry events sent before this happens
+            # will result in an uninitialized container id value.
+
             logger.info(u"Agent {0} is running as the goal state agent",
                         CURRENT_AGENT)
 
+            # Log OS-specific info locally.
+            os_info_msg = u"Distro info: {0} {1}, osutil class being used: {2}, " \
+                          u"agent service name: {3}".format(DISTRO_NAME, DISTRO_VERSION,
+                                                            type(self.osutil).__name__, self.osutil.service_name)
+            logger.info(os_info_msg)
+
             # Launch monitoring threads
             from azurelinuxagent.ga.monitor import get_monitor_handler
             monitor_thread = get_monitor_handler()
             monitor_thread.run()
 
+            # NOTE: Any telemetry events added from this point on will be properly populated with the container id.
+
             from azurelinuxagent.ga.env import get_env_handler
             env_thread = get_env_handler()
             env_thread.run()
@@ -274,13 +284,20 @@ class UpdateHandler(object):
             self._emit_restart_event()
             self._ensure_partition_assigned()
             self._ensure_readonly_files()
+            self._ensure_cgroups_initialized()
+
+            # Send OS-specific info as a telemetry event after the monitoring thread has been initialized, and with
+            # it the container id too.
+            add_event(AGENT_NAME,
+                      op=WALAEventOperation.OSInfo,
+                      message=os_info_msg)
 
             goal_state_interval = GOAL_STATE_INTERVAL \
                 if conf.get_extensions_enabled() \
                 else GOAL_STATE_INTERVAL_DISABLED
 
             while self.running:
-                if self._is_orphaned:
+                if not debug and self._is_orphaned:
                     logger.info("Agent {0} is an orphan -- exiting",
                                 CURRENT_AGENT)
                     break
@@ -447,6 +464,12 @@ class UpdateHandler(object):
             for path in glob.iglob(os.path.join(conf.get_lib_dir(), g)):
                 os.chmod(path, stat.S_IRUSR)
 
+    def _ensure_cgroups_initialized(self):
+        configurator = CGroupConfigurator.get_instance()
+        configurator.create_agent_cgroups(track_cgroups=True)
+        configurator.cleanup_legacy_cgroups()
+        configurator.create_extension_cgroups_root()
+
     def _evaluate_agent_health(self, latest_agent):
         """
         Evaluate the health of the selected agent: If it is restarting
@@ -491,10 +514,7 @@ class UpdateHandler(object):
 
     def _get_host_plugin(self, protocol=None):
         return protocol.client.get_host_plugin() \
-                                if protocol and \
-                                    type(protocol) is WireProtocol and \
-                                    protocol.client \
-                                else None
+            if protocol and type(protocol) is WireProtocol and protocol.client else None
 
     def _get_pid_parts(self):
         pid_file = conf.get_agent_pid_file_path()
@@ -640,58 +660,48 @@ class UpdateHandler(object):
         self.last_attempt_time = now
         protocol = self.protocol_util.get_protocol()
 
-        for update_goal_state in [False, True]:
-            try:
-                if update_goal_state:
-                    protocol.update_goal_state(forced=True)
+        try:
+            manifest_list, etag = protocol.get_vmagent_manifests()
 
-                manifest_list, etag = protocol.get_vmagent_manifests()
-
-                manifests = [m for m in manifest_list.vmAgentManifests \
-                                if m.family == family and \
-                                    len(m.versionsManifestUris) > 0]
-                if len(manifests) == 0:
-                    logger.verbose(u"Incarnation {0} has no {1} agent updates",
-                                    etag, family)
-                    return False
-
-                pkg_list = protocol.get_vmagent_pkgs(manifests[0])
-
-                # Set the agents to those available for download at least as
-                # current as the existing agent and remove from disk any agent
-                # no longer reported to the VM.
-                # Note:
-                #  The code leaves on disk available, but blacklisted, agents
-                #  so as to preserve the state. Otherwise, those agents could be
-                #  again downloaded and inappropriately retried.
-                host = self._get_host_plugin(protocol=protocol)
-                self._set_agents([GuestAgent(pkg=pkg, host=host) \
-                                     for pkg in pkg_list.versions])
-
-                self._purge_agents()
-                self._filter_blacklisted_agents()
-
-                # Return True if current agent is no longer available or an
-                # agent with a higher version number is available
-                return not self._is_version_eligible(base_version) \
-                    or (len(self.agents) > 0 \
-                        and self.agents[0].version > base_version)
-
-            except Exception as e:
-                if isinstance(e, ResourceGoneError):
-                    continue
-
-                msg = u"Exception retrieving agent manifests: {0}".format(
-                            ustr(traceback.format_exc()))
-                logger.warn(msg)
-                add_event(
-                    AGENT_NAME,
-                    op=WALAEventOperation.Download,
-                    version=CURRENT_VERSION,
-                    is_success=False,
-                    message=msg)
+            manifests = [m for m in manifest_list.vmAgentManifests \
+                            if m.family == family and len(m.versionsManifestUris) > 0]
+            if len(manifests) == 0:
+                logger.verbose(u"Incarnation {0} has no {1} agent updates",
+                                etag, family)
                 return False
 
+            pkg_list = protocol.get_vmagent_pkgs(manifests[0])
+
+            # Set the agents to those available for download at least as
+            # current as the existing agent and remove from disk any agent
+            # no longer reported to the VM.
+            # Note:
+            #  The code leaves on disk available, but blacklisted, agents
+            #  so as to preserve the state. Otherwise, those agents could be
+            #  again downloaded and inappropriately retried.
+            host = self._get_host_plugin(protocol=protocol)
+            self._set_agents([GuestAgent(pkg=pkg, host=host) for pkg in pkg_list.versions])
+
+            self._purge_agents()
+            self._filter_blacklisted_agents()
+
+            # Return True if current agent is no longer available or an
+            # agent with a higher version number is available
+            return not self._is_version_eligible(base_version) \
+                or (len(self.agents) > 0 and self.agents[0].version > base_version)
+
+        except Exception as e:
+            msg = u"Exception retrieving agent manifests: {0}".format(
+                        ustr(traceback.format_exc()))
+            logger.warn(msg)
+            add_event(
+                AGENT_NAME,
+                op=WALAEventOperation.Download,
+                version=CURRENT_VERSION,
+                is_success=False,
+                message=msg)
+            return False
+
     def _write_pid_file(self):
         pid_files = self._get_pid_files()
 
