@@ -16,7 +16,8 @@
 #
 from azurelinuxagent.common.osutil.clearlinux import ClearLinuxUtil
 from .test_default import osutil_get_dhcp_pid_should_return_a_list_of_pids
-from tests.tools import *
+from tests.tools import AgentTestCase
+import unittest
 
 
 class TestClearLinuxUtil(AgentTestCase):
