@@ -14,12 +14,17 @@
 #
 # Requires Python 2.6+ and Openssl 1.0+
 #
+
+import os
+import unittest
+
 from multiprocessing import Process
 
+import azurelinuxagent.common.conf as conf
 from azurelinuxagent.daemon import *
 from azurelinuxagent.daemon.main import OPENSSL_FIPS_ENVIRONMENT
 from azurelinuxagent.pa.provision.default import ProvisionHandler
-from tests.tools import *
+from tests.tools import AgentTestCase, Mock, patch
 
 
 class MockDaemonCall(object):
@@ -85,49 +90,50 @@ class TestDaemon(AgentTestCase):
             daemon_handler.run()
             self.assertFalse(OPENSSL_FIPS_ENVIRONMENT in os.environ)
 
+    @patch('azurelinuxagent.common.conf.get_provisioning_agent', return_value='waagent')
     @patch('azurelinuxagent.ga.update.UpdateHandler.run_latest')
     @patch('azurelinuxagent.pa.provision.default.ProvisionHandler.run')
-    @patch('azurelinuxagent.pa.provision.get_provision_handler', return_value=ProvisionHandler())
-    def test_daemon_agent_enabled(self, _, patch_run_provision, patch_run_latest):
+    def test_daemon_agent_enabled(self, patch_run_provision, patch_run_latest, gpa):
         """
         Agent should run normally when no disable_agent is found
         """
+        with patch('azurelinuxagent.pa.provision.get_provision_handler', return_value=ProvisionHandler()):
+            self.assertFalse(os.path.exists(conf.get_disable_agent_file_path()))
+            daemon_handler = get_daemon_handler()
 
-        self.assertFalse(os.path.exists(conf.get_disable_agent_file_path()))
-        daemon_handler = get_daemon_handler()
+            def stop_daemon(child_args):
+                daemon_handler.running = False
 
-        def stop_daemon(child_args):
-            daemon_handler.running = False
+            patch_run_latest.side_effect = stop_daemon
+            daemon_handler.run()
 
-        patch_run_latest.side_effect = stop_daemon
-        daemon_handler.run()
-
-        self.assertEqual(1, patch_run_provision.call_count)
-        self.assertEqual(1, patch_run_latest.call_count)
+            self.assertEqual(1, patch_run_provision.call_count)
+            self.assertEqual(1, patch_run_latest.call_count)
 
+    @patch('azurelinuxagent.common.conf.get_provisioning_agent', return_value='waagent')
     @patch('azurelinuxagent.ga.update.UpdateHandler.run_latest', side_effect=AgentTestCase.fail)
     @patch('azurelinuxagent.pa.provision.default.ProvisionHandler.run', side_effect=ProvisionHandler.write_agent_disabled)
-    @patch('azurelinuxagent.pa.provision.get_provision_handler', return_value=ProvisionHandler())
-    def test_daemon_agent_disabled(self, _, __, patch_run_latest):
+    def test_daemon_agent_disabled(self, _, patch_run_latest, gpa):
         """
         Agent should provision, then sleep forever when disable_agent is found
         """
 
-        # file is created by provisioning handler
-        self.assertFalse(os.path.exists(conf.get_disable_agent_file_path()))
-        daemon_handler = get_daemon_handler()
+        with patch('azurelinuxagent.pa.provision.get_provision_handler', return_value=ProvisionHandler()):
+            # file is created by provisioning handler
+            self.assertFalse(os.path.exists(conf.get_disable_agent_file_path()))
+            daemon_handler = get_daemon_handler()
 
-        # we need to assert this thread will sleep forever, so fork it
-        daemon = Process(target=daemon_handler.run)
-        daemon.start()
-        daemon.join(timeout=5)
+            # we need to assert this thread will sleep forever, so fork it
+            daemon = Process(target=daemon_handler.run)
+            daemon.start()
+            daemon.join(timeout=5)
 
-        self.assertTrue(daemon.is_alive())
-        daemon.terminate()
+            self.assertTrue(daemon.is_alive())
+            daemon.terminate()
 
-        # disable_agent was written, run_latest was not called
-        self.assertTrue(os.path.exists(conf.get_disable_agent_file_path()))
-        self.assertEqual(0, patch_run_latest.call_count)
+            # disable_agent was written, run_latest was not called
+            self.assertTrue(os.path.exists(conf.get_disable_agent_file_path()))
+            self.assertEqual(0, patch_run_latest.call_count)
 
 
 if __name__ == '__main__':
