@@ -26,27 +26,33 @@ import tempfile
 import time
 from datetime import timedelta
 
-from mock import Mock, MagicMock, patch
 from nose.plugins.attrib import attr
 
-from azurelinuxagent.common import logger
-from azurelinuxagent.common.cgroup import CGroup
-from azurelinuxagent.common.cgroupstelemetry import CGroupsTelemetry
+import azurelinuxagent.common.conf as conf
+from azurelinuxagent.common import event, logger
+from azurelinuxagent.common.cgroup import CGroup, CpuCgroup, MemoryCgroup
+from azurelinuxagent.common.cgroupconfigurator import CGroupConfigurator
+from azurelinuxagent.common.cgroupstelemetry import CGroupsTelemetry, MetricValue
 from azurelinuxagent.common.datacontract import get_properties
-from azurelinuxagent.common.event import EventLogger, WALAEventOperation, CONTAINER_ID_ENV_VARIABLE
+from azurelinuxagent.common.event import CONTAINER_ID_ENV_VARIABLE, EventLogger, WALAEventOperation
 from azurelinuxagent.common.exception import HttpError
 from azurelinuxagent.common.future import ustr
+from azurelinuxagent.common.logger import Logger
+from azurelinuxagent.common.osutil.default import BASE_CGROUPS, DefaultOSUtil
 from azurelinuxagent.common.protocol.imds import ComputeInfo
 from azurelinuxagent.common.protocol.restapi import VMInfo
+from azurelinuxagent.common.protocol.wire import ExtHandler, ExtHandlerProperties
 from azurelinuxagent.common.protocol.wire import WireProtocol
-from azurelinuxagent.common.telemetryevent import TelemetryEventParam, TelemetryEvent
-from azurelinuxagent.common.utils import restutil, fileutil
-from azurelinuxagent.common.version import AGENT_VERSION, CURRENT_VERSION, AGENT_NAME, CURRENT_AGENT
-from azurelinuxagent.ga.monitor import parse_xml_event, get_monitor_handler, MonitorHandler, \
-    generate_extension_metrics_telemetry_dictionary, parse_json_event
-from tests.common.test_cgroupstelemetry import make_new_cgroup, consume_cpu_time, consume_memory
-from tests.protocol.mockwiredata import WireProtocolData, DATA_FILE, conf
-from tests.tools import load_data, AgentTestCase, data_dir, are_cgroups_enabled, i_am_root, skip_if_predicate_false
+from azurelinuxagent.common.telemetryevent import TelemetryEvent, TelemetryEventParam
+from azurelinuxagent.common.utils import fileutil, restutil
+from azurelinuxagent.common.version import AGENT_NAME, CURRENT_VERSION, AGENT_VERSION, CURRENT_AGENT
+from azurelinuxagent.ga.exthandlers import ExtHandlerInstance
+from azurelinuxagent.ga.monitor import generate_extension_metrics_telemetry_dictionary, get_monitor_handler, \
+    MonitorHandler, parse_json_event, parse_xml_event
+from tests.common.test_cgroupstelemetry import make_new_cgroup
+from tests.protocol.mockwiredata import DATA_FILE, WireProtocolData
+from tests.tools import Mock, MagicMock, patch, load_data, AgentTestCase, data_dir, are_cgroups_enabled, \
+    i_am_root, skip_if_predicate_false, is_trusty_in_travis, skip_if_predicate_true
 
 
 class ResponseMock(Mock):
@@ -107,6 +113,13 @@ def get_event_message(duration, evt_type, is_internal, is_success, message, name
 @patch("azurelinuxagent.common.protocol.healthservice.HealthService._report")
 @patch("azurelinuxagent.common.utils.restutil.http_get")
 class TestMonitor(AgentTestCase):
+    def setUp(self):
+        AgentTestCase.setUp(self)
+        prefix = "UnitTest"
+        logger.DEFAULT_LOGGER = Logger(prefix=prefix)
+
+    def tearDown(self):
+        AgentTestCase.tearDown(self)
 
     def test_parse_xml_event(self, *args):
         data_str = load_data('ext/event_from_extension.xml')
@@ -554,8 +567,9 @@ class TestEventMonitoring(AgentTestCase):
                                                        "DISTRO_CODE_NAME",
                                                        platform.release())
 
+    @patch("azurelinuxagent.common.event.send_logs_to_telemetry", return_value=True)
     @patch("azurelinuxagent.common.conf.get_lib_dir")
-    def test_collect_and_send_events_should_prepare_all_fields_for_all_event_files(self, mock_lib_dir, *args):
+    def test_collect_and_send_events_should_prepare_all_fields_for_all_event_files(self, mock_lib_dir, _, *args):
         # Test collecting and sending both agent and extension events from the moment they're created to the moment
         # they are to be reported. Ensure all necessary fields from sysinfo are present, as well as the container id.
         mock_lib_dir.return_value = self.lib_dir
@@ -573,6 +587,12 @@ class TestEventMonitoring(AgentTestCase):
                                     message="Heartbeat",
                                     log_event=False)
 
+        # Add agent metric
+        self.event_logger.add_metric("Process", "% Processor Time", "walinuxagent.service", 10)
+
+        # Add agent log
+        self.event_logger.add_log_event(logger.LogLevel.WARNING, "Test sending a log event.")
+
         # Add extension event file the way extension do it, by dropping a .tld file in the events folder
         source_file = os.path.join(data_dir, "ext/dsc_event.json")
         dest_file = os.path.join(conf.get_lib_dir(), "events", "dsc_event.tld")
@@ -583,18 +603,24 @@ class TestEventMonitoring(AgentTestCase):
             monitor_handler.collect_and_send_events()
 
             telemetry_events_list = patch_report_event.call_args_list[0][0][0]
-            self.assertEqual(len(telemetry_events_list.events), 2)
+            self.assertEqual(len(telemetry_events_list.events), 4)
 
             for event in telemetry_events_list.events:
                 # All sysinfo parameters coming from the agent have to be present in the telemetry event to be emitted
                 for param in monitor_handler.sysinfo:
                     self.assertTrue(param in event.parameters)
 
-                # The container id is a special parameter that is not a part of the static sysinfo parameter list.
+                # The container id, GAVersion are special parameters that are not a part of the static sysinfo parameter
+                # list.
+
                 # The container id value is obtained from the goal state and must be present in all telemetry events.
                 container_id_param = TelemetryEventParam("ContainerId", protocol.client.goal_state.container_id)
                 self.assertTrue(container_id_param in event.parameters)
 
+                # Same for GAVersion
+                container_id_param = TelemetryEventParam("GAVersion", CURRENT_AGENT)
+                self.assertTrue(container_id_param in event.parameters)
+
     @patch("azurelinuxagent.common.protocol.wire.WireClient.send_event")
     @patch("azurelinuxagent.common.conf.get_lib_dir")
     def test_collect_and_send_events(self, mock_lib_dir, patch_send_event, *args):
@@ -613,8 +639,9 @@ class TestEventMonitoring(AgentTestCase):
         # Validating the crafted message by the collect_and_send_events call.
         self.assertEqual(1, patch_send_event.call_count)
         send_event_call_args = protocol.client.send_event.call_args[0]
-        sample_message = '<Event id="1">' \
-                         '<![CDATA[<Param Name="Name" Value="DummyExtension" T="mt:wstr" />' \
+
+        sample_message = '<Event id="1"><![CDATA[' \
+                         '<Param Name="Name" Value="DummyExtension" T="mt:wstr" />' \
                          '<Param Name="Version" Value="{0}" T="mt:wstr" />' \
                          '<Param Name="IsInternal" Value="False" T="mt:bool" />' \
                          '<Param Name="Operation" Value="Unknown" T="mt:wstr" />' \
@@ -637,13 +664,13 @@ class TestEventMonitoring(AgentTestCase):
                          '<Param Name="ResourceGroupName" Value="DummyRG" T="mt:wstr" />' \
                          '<Param Name="VMId" Value="DummyVmId" T="mt:wstr" />' \
                          '<Param Name="ImageOrigin" Value="1" T="mt:uint64" />' \
-                         '<Param Name="ContainerId" Value="c6d5526c-5ac2-4200-b6e2-56f2b70c5ab2" T="mt:wstr" />' \
                          '<Param Name="GAVersion" Value="{1}" T="mt:wstr" />' \
+                         '<Param Name="ContainerId" Value="c6d5526c-5ac2-4200-b6e2-56f2b70c5ab2" T="mt:wstr" />' \
                          '<Param Name="EventTid" Value="0" T="mt:uint64" />' \
                          '<Param Name="EventPid" Value="0" T="mt:uint64" />' \
                          '<Param Name="TaskName" Value="" T="mt:wstr" />' \
-                         '<Param Name="KeywordName" Value="" T="mt:wstr" />]]>' \
-                         '</Event>'.format(AGENT_VERSION, CURRENT_AGENT)
+                         '<Param Name="KeywordName" Value="" T="mt:wstr" />' \
+                         ']]></Event>'.format(AGENT_VERSION, CURRENT_AGENT)
 
         self.maxDiff = None
         self.assertEqual(sample_message, send_event_call_args[1])
@@ -730,7 +757,7 @@ class TestEventMonitoring(AgentTestCase):
                 return "builtins"
 
         with patch("{0}.open".format(builtins_version())) as mock_open:
-            mock_open.side_effect = OSError(13, "Permission denied")
+            mock_open.side_effect = IOError(13, "Permission denied")
             monitor_handler.collect_and_send_events()
 
             # Invalid events
@@ -815,20 +842,30 @@ class TestEventMonitoring(AgentTestCase):
 
 @patch('azurelinuxagent.common.osutil.get_osutil')
 @patch('azurelinuxagent.common.protocol.get_protocol_util')
-@patch('azurelinuxagent.common.protocol.util.ProtocolUtil.get_protocol')
 @patch("azurelinuxagent.common.protocol.healthservice.HealthService._report")
+@patch('azurelinuxagent.common.protocol.util.ProtocolUtil.get_protocol', return_value=WireProtocol('endpoint'))
 @patch("azurelinuxagent.common.utils.restutil.http_get")
 class TestExtensionMetricsDataTelemetry(AgentTestCase):
 
     def setUp(self):
         AgentTestCase.setUp(self)
+        event.init_event_logger(os.path.join(self.tmp_dir, "events"))
         CGroupsTelemetry.reset()
 
+    def tearDown(self):
+        AgentTestCase.tearDown(self)
+        CGroupsTelemetry.reset()
+
+    @patch('azurelinuxagent.common.event.EventLogger.add_metric')
     @patch('azurelinuxagent.common.event.EventLogger.add_event')
     @patch("azurelinuxagent.common.cgroupstelemetry.CGroupsTelemetry.poll_all_tracked")
     @patch("azurelinuxagent.common.cgroupstelemetry.CGroupsTelemetry.report_all_tracked")
     def test_send_extension_metrics_telemetry(self, patch_report_all_tracked, patch_poll_all_tracked, patch_add_event,
-                                              *args):
+                                              patch_add_metric, *args):
+        patch_poll_all_tracked.return_value = [MetricValue("Process", "% Processor Time", 1, 1),
+                                               MetricValue("Memory", "Total Memory Usage", 1, 1),
+                                               MetricValue("Memory", "Max Memory Usage", 1, 1)]
+
         patch_report_all_tracked.return_value = {
             "memory": {
                 "cur_mem": [1, 1, 1, 1, 1, str(datetime.datetime.utcnow()), str(datetime.datetime.utcnow())],
@@ -848,14 +885,17 @@ class TestExtensionMetricsDataTelemetry(AgentTestCase):
         self.assertEqual(1, patch_poll_all_tracked.call_count)
         self.assertEqual(1, patch_report_all_tracked.call_count)
         self.assertEqual(1, patch_add_event.call_count)
+        self.assertEqual(3, patch_add_metric.call_count)  # Three metrics being sent.
         monitor_handler.stop()
 
+    @patch('azurelinuxagent.common.event.EventLogger.add_metric')
     @patch('azurelinuxagent.common.event.EventLogger.add_event')
     @patch("azurelinuxagent.common.cgroupstelemetry.CGroupsTelemetry.poll_all_tracked")
     @patch("azurelinuxagent.common.cgroupstelemetry.CGroupsTelemetry.report_all_tracked", return_value={})
     def test_send_extension_metrics_telemetry_for_empty_cgroup(self, patch_report_all_tracked, patch_poll_all_tracked,
-                                                               patch_add_event, *args):
+                                                               patch_add_event, patch_add_metric,*args):
         patch_report_all_tracked.return_value = {}
+        patch_poll_all_tracked.return_value = []
 
         monitor_handler = get_monitor_handler()
         monitor_handler.init_protocols()
@@ -866,50 +906,248 @@ class TestExtensionMetricsDataTelemetry(AgentTestCase):
         self.assertEqual(1, patch_poll_all_tracked.call_count)
         self.assertEqual(1, patch_report_all_tracked.call_count)
         self.assertEqual(0, patch_add_event.call_count)
+        self.assertEqual(0, patch_add_metric.call_count)
         monitor_handler.stop()
 
-    @skip_if_predicate_false(are_cgroups_enabled, "Does not run when Cgroups are not enabled")
-    @patch('azurelinuxagent.common.event.EventLogger.add_event')
-    @attr('requires_sudo')
-    def test_send_extension_metrics_telemetry_with_actual_cgroup(self, patch_add_event, *args):
-        self.assertTrue(i_am_root(), "Test does not run when non-root")
+    @patch('azurelinuxagent.common.event.EventLogger.add_metric')
+    @patch("azurelinuxagent.common.cgroup.MemoryCgroup.get_memory_usage")
+    @patch('azurelinuxagent.common.logger.Logger.periodic_warn')
+    def test_send_extension_metrics_telemetry_handling_memory_cgroup_exceptions_errno2(self, patch_periodic_warn,
+                                                                                       patch_get_memory_usage,
+                                                                                       patch_add_metric, *args):
+        ioerror = IOError()
+        ioerror.errno = 2
+        patch_get_memory_usage.side_effect = ioerror
 
-        num_polls = 5
-        name = "test-cgroup"
-
-        cgs = make_new_cgroup(name)
-
-        self.assertEqual(len(cgs), 2)
-
-        for cgroup in cgs:
-            CGroupsTelemetry.track_cgroup(cgroup)
-
-        for i in range(num_polls):
-            CGroupsTelemetry.poll_all_tracked()
-            consume_cpu_time()  # Eat some CPU
-            consume_memory()
+        CGroupsTelemetry._tracked.append(MemoryCgroup("cgroup_name", "/test/path"))
 
         monitor_handler = get_monitor_handler()
         monitor_handler.init_protocols()
         monitor_handler.last_cgroup_polling_telemetry = datetime.datetime.utcnow() - timedelta(hours=1)
         monitor_handler.last_cgroup_report_telemetry = datetime.datetime.utcnow() - timedelta(hours=1)
         monitor_handler.poll_telemetry_metrics()
-        monitor_handler.send_telemetry_metrics()
-        self.assertEqual(1, patch_add_event.call_count)
+        self.assertEqual(0, patch_periodic_warn.call_count)
+        self.assertEqual(0, patch_add_metric.call_count)  # No metrics should be sent.
+        monitor_handler.stop()
 
-        name = patch_add_event.call_args[0][0]
-        fields = patch_add_event.call_args[1]
+    @patch('azurelinuxagent.common.event.EventLogger.add_metric')
+    @patch("azurelinuxagent.common.cgroup.CpuCgroup.get_cpu_usage")
+    @patch('azurelinuxagent.common.logger.Logger.periodic_warn')
+    def test_send_extension_metrics_telemetry_handling_cpu_cgroup_exceptions_errno2(self, patch_periodic_warn,
+                                                                                    patch_cpu_usage, patch_add_metric,
+                                                                                    *args):
+        ioerror = IOError()
+        ioerror.errno = 2
+        patch_cpu_usage.side_effect = ioerror
 
-        self.assertEqual(name, "WALinuxAgent")
-        self.assertEqual(fields["op"], "ExtensionMetricsData")
-        self.assertEqual(fields["is_success"], True)
-        self.assertEqual(fields["log_event"], False)
-        self.assertEqual(fields["is_internal"], False)
-        self.assertIsInstance(fields["message"], ustr)
+        CGroupsTelemetry._tracked.append(CpuCgroup("cgroup_name", "/test/path"))
+
+        monitor_handler = get_monitor_handler()
+        monitor_handler.init_protocols()
+        monitor_handler.last_cgroup_polling_telemetry = datetime.datetime.utcnow() - timedelta(hours=1)
+        monitor_handler.last_cgroup_report_telemetry = datetime.datetime.utcnow() - timedelta(hours=1)
+        monitor_handler.poll_telemetry_metrics()
+        self.assertEqual(0, patch_periodic_warn.call_count)
+        self.assertEqual(0, patch_add_metric.call_count)  # No metrics should be sent.
+        monitor_handler.stop()
+
+    @patch('azurelinuxagent.common.event.EventLogger.add_metric')
+    @patch('azurelinuxagent.common.logger.Logger.periodic_warn')
+    def test_send_extension_metrics_telemetry_for_unsupported_cgroup(self, patch_periodic_warn, patch_add_metric, *args):
+        CGroupsTelemetry._tracked.append(CGroup("cgroup_name", "/test/path", "io"))
+
+        monitor_handler = get_monitor_handler()
+        monitor_handler.init_protocols()
+        monitor_handler.last_cgroup_polling_telemetry = datetime.datetime.utcnow() - timedelta(hours=1)
+        monitor_handler.last_cgroup_report_telemetry = datetime.datetime.utcnow() - timedelta(hours=1)
+        monitor_handler.poll_telemetry_metrics()
+        self.assertEqual(1, patch_periodic_warn.call_count)
+        self.assertEqual(0, patch_add_metric.call_count)  # No metrics should be sent.
+
+        monitor_handler.stop()
+
+    @skip_if_predicate_true(lambda: True, "Skipping this test currently: We need two different tests - one for "
+                                  "FileSystemCgroupAPI based test and one for SystemDCgroupAPI based test. @vrdmr will "
+                                  "be splitting this test in subsequent PRs")
+    @skip_if_predicate_false(are_cgroups_enabled, "Does not run when Cgroups are not enabled")
+    @skip_if_predicate_true(is_trusty_in_travis, "Does not run on Trusty in Travis")
+    @patch('azurelinuxagent.common.event.EventLogger.add_metric')
+    @patch('azurelinuxagent.common.event.EventLogger.add_event')
+    @attr('requires_sudo')
+    def test_send_extension_metrics_telemetry_with_actual_cgroup(self, patch_add_event, patch_add_metric, *arg):
+        self.assertTrue(i_am_root(), "Test does not run when non-root")
+
+        # This test has some timing issues when systemd is managing cgroups, so we force the file system API
+        # by creating a new instance of the CGroupConfigurator
+        with patch("azurelinuxagent.common.cgroupapi.CGroupsApi._is_systemd", return_value=False):
+            cgroup_configurator_instance = CGroupConfigurator._instance
+            CGroupConfigurator._instance = None
+
+            try:
+                max_num_polls = 5
+                time_to_wait = 3
+                extn_name = "foobar-1.0.0"
+
+                cgs = make_new_cgroup(extn_name)
+                self.assertEqual(len(cgs), 2)
+
+                ext_handler_properties = ExtHandlerProperties()
+                ext_handler_properties.version = "1.0.0"
+                ext_handler = ExtHandler(name='foobar')
+                ext_handler.properties = ext_handler_properties
+                ext_handler_instance = ExtHandlerInstance(ext_handler=ext_handler, protocol=None)
+                ext_handler_instance.set_operation("Enable")
+
+                monitor_handler = get_monitor_handler()
+                monitor_handler.init_protocols()
+                monitor_handler.last_cgroup_polling_telemetry = datetime.datetime.utcnow() - timedelta(hours=1)
+                monitor_handler.last_cgroup_report_telemetry = datetime.datetime.utcnow() - timedelta(hours=1)
+
+                command = self.create_script("keep_cpu_busy_and_consume_memory_for_{0}_seconds".format(time_to_wait), '''
+nohup python -c "import time
+
+for i in range(5):
+    x = [1, 2, 3, 4, 5] * (i * 1000)
+    time.sleep({0})
+    x = [1, 2, 3, 4, 5] * (i * 1000)
+    x *= 0
+    print('Test loop')" &
+'''.format(time_to_wait))
+
+                self.log_dir = os.path.join(self.tmp_dir, "log")
+
+                with patch("azurelinuxagent.ga.exthandlers.ExtHandlerInstance.get_base_dir", lambda *_: self.tmp_dir) as \
+                        patch_get_base_dir:
+                    with patch("azurelinuxagent.ga.exthandlers.ExtHandlerInstance.get_log_dir", lambda *_: self.log_dir) as \
+                            patch_get_log_dir:
+                        ext_handler_instance.launch_command(command)
+
+                self.assertTrue(CGroupsTelemetry.is_tracked(os.path.join(
+                    BASE_CGROUPS, "cpu", "walinuxagent.extensions", "foobar_1.0.0")))
+                self.assertTrue(CGroupsTelemetry.is_tracked(os.path.join(
+                    BASE_CGROUPS, "memory", "walinuxagent.extensions", "foobar_1.0.0")))
+
+                for i in range(max_num_polls):
+                    metrics = CGroupsTelemetry.poll_all_tracked()
+                    self.assertEqual(len(metrics), 3)
+
+                monitor_handler.poll_telemetry_metrics()
+                self.assertEqual(3, patch_add_metric.call_count)
+
+                for call_arg in patch_add_metric.call_args_list:
+                    self.assertIn(call_arg[0][0], ["Process", "Memory"])
+                    if call_arg[0][0] == "Process":
+                        self.assertEqual(call_arg[0][1], "% Processor Time")
+                    if call_arg[0][0] == "Memory":
+                        self.assertIn(call_arg[0][1], ["Total Memory Usage", "Max Memory Usage"])
+                    self.assertIsInstance(call_arg[0][3], float)
+
+                    self.assertEqual(call_arg[0][2], extn_name)
+                    self.assertFalse(call_arg[0][4])
+
+                monitor_handler.send_telemetry_metrics()
+                self.assertEqual(3, patch_add_event.call_count)     # 1 for launch command, 1 for extension metrics data
+                                                                    # and 1 for Cgroups initialization
+                name = patch_add_event.call_args[0][0]
+                fields = patch_add_event.call_args[1]
+
+                self.assertEqual(name, "WALinuxAgent")
+                self.assertEqual(fields["op"], "ExtensionMetricsData")
+                self.assertEqual(fields["is_success"], True)
+                self.assertEqual(fields["log_event"], False)
+                self.assertEqual(fields["is_internal"], False)
+                self.assertIsInstance(fields["message"], ustr)
+                monitor_handler.stop()
+            finally:
+                CGroupConfigurator._instance = cgroup_configurator_instance
+
+    @skip_if_predicate_true(lambda: True, "Skipping this test currently: We need two different tests - one for "
+                                  "FileSystemCgroupAPI based test and one for SystemDCgroupAPI based test. @vrdmr will "
+                                  "be splitting this test in subsequent PRs")
+    @skip_if_predicate_false(are_cgroups_enabled, "Does not run when Cgroups are not enabled")
+    @skip_if_predicate_true(is_trusty_in_travis, "Does not run on Trusty in Travis")
+    @patch("azurelinuxagent.common.cgroupconfigurator.get_osutil", return_value=DefaultOSUtil())
+    @patch("azurelinuxagent.common.cgroupapi.CGroupsApi._is_systemd", return_value=False)
+    @patch('azurelinuxagent.common.protocol.wire.WireClient.report_event')
+    @attr('requires_sudo')
+    def test_report_event_metrics_sent_for_actual_cgroup(self, patch_report_event, patch__is_systemd, patch_get_osutil,
+                                                         http_get, patch_get_protocol, *args):
+        self.assertTrue(i_am_root(), "Test does not run when non-root")
+        CGroupConfigurator._instance = None
+
+        max_num_polls = 5
+        time_to_wait = 1
+        extn_name = "foobar-1.0.0"
+        extn_folder_name = extn_name.replace("-", "_")
+
+        cgs = make_new_cgroup(extn_name)
+        self.assertEqual(len(cgs), 2)
+
+        ext_handler_properties = ExtHandlerProperties()
+        ext_handler_properties.version = "1.0.0"
+        ext_handler = ExtHandler(name='foobar')
+        ext_handler.properties = ext_handler_properties
+        ext_handler_instance = ExtHandlerInstance(ext_handler=ext_handler, protocol=None)
+        ext_handler_instance.set_operation("Enable")
+
+        monitor_handler = get_monitor_handler()
+        monitor_handler.init_protocols()
+        monitor_handler.last_cgroup_polling_telemetry = datetime.datetime.utcnow() - timedelta(hours=1)
+        monitor_handler.last_cgroup_report_telemetry = datetime.datetime.utcnow() - timedelta(hours=1)
+
+        command = self.create_script("keep_cpu_busy_and_consume_memory_for_{0}_seconds".format(time_to_wait), '''
+nohup python -c "import time
+import subprocess
+
+for i in range(3):
+    x = [1, 2, 3, 4, 5] * (i * 1000)
+    time.sleep({0})
+    x *= 0
+    print('Test loop')
+
+" &
+'''.format(time_to_wait))
+
+        self.log_dir = os.path.join(self.tmp_dir, "log")
+
+        with patch("azurelinuxagent.ga.exthandlers.ExtHandlerInstance.get_base_dir", lambda *_: self.tmp_dir):
+            with patch("azurelinuxagent.ga.exthandlers.ExtHandlerInstance.get_log_dir", lambda *_: self.log_dir):
+                ext_handler_instance.launch_command(command)
+
+        self.assertTrue(CGroupsTelemetry.is_tracked(os.path.join(BASE_CGROUPS, "cpu", "walinuxagent.extensions", extn_folder_name)))
+        self.assertTrue(CGroupsTelemetry.is_tracked(os.path.join(BASE_CGROUPS, "memory", "walinuxagent.extensions", extn_folder_name)))
+
+        for i in range(max_num_polls):
+            metrics = CGroupsTelemetry.poll_all_tracked()
+            # Currently there are 3 types of memory related metrics and 1 CPU related metric.
+            # % Processor Time
+            # Total Memory Usage
+            # Max Memory Usage
+            # Memory Used by Process - This can have multiple entries (for each process that gets created).
+            self.assertEqual(len(metrics), 4)
+
+        monitor_handler.poll_telemetry_metrics()
+        monitor_handler.send_telemetry_metrics()
+        monitor_handler.collect_and_send_events()
+
+        telemetry_event_list = patch_report_event.call_args_list[0][0][0]
+
+        for e in telemetry_event_list.events:
+            print([(i.name, i.value) for i in e.parameters])
+            details_of_event = [x for x in e.parameters if x.name in
+                                ["Category", "Counter", "Instance", "Value"]]
+
+            for i in details_of_event:
+                if i.name == "Category":
+                    self.assertIn(i.value, ["Memory", "Process"])
+                if i.name == "Counter":
+                    self.assertIn(i.value, ["Max Memory Usage", "Total Memory Usage", "% Processor Time",
+                                            "Memory Used by Process"])
+                if i.name == "Value":
+                    self.assertTrue(isinstance(i.value, int) or isinstance(i.value, float))
 
         monitor_handler.stop()
 
-    @patch("azurelinuxagent.common.osutil.default.DefaultOSUtil._get_proc_stat")
     def test_generate_extension_metrics_telemetry_dictionary(self, *args):
         num_polls = 10
         num_extensions = 1
@@ -921,27 +1159,28 @@ class TestExtensionMetricsDataTelemetry(AgentTestCase):
         memory_usage_values = [random.randint(0, 8 * 1024 ** 3) for _ in range(num_polls)]
         max_memory_usage_values = [random.randint(0, 8 * 1024 ** 3) for _ in range(num_polls)]
 
-        for i in range(num_extensions):
-            dummy_cpu_cgroup = CGroup.create("dummy_cpu_path_{0}".format(i), "cpu", "dummy_extension_{0}".format(i))
-            CGroupsTelemetry.track_cgroup(dummy_cpu_cgroup)
+        # no need to initialize the CPU usage, since we mock get_cpu_usage() below
+        with patch("azurelinuxagent.common.cgroup.CpuCgroup.initialize_cpu_usage"):
+            for i in range(num_extensions):
+                dummy_cpu_cgroup = CGroup.create("dummy_cpu_path_{0}".format(i), "cpu", "dummy_extension_{0}".format(i))
+                CGroupsTelemetry.track_cgroup(dummy_cpu_cgroup)
 
-            dummy_memory_cgroup = CGroup.create("dummy_memory_path_{0}".format(i), "memory",
-                                                "dummy_extension_{0}".format(i))
-            CGroupsTelemetry.track_cgroup(dummy_memory_cgroup)
+                dummy_memory_cgroup = CGroup.create("dummy_memory_path_{0}".format(i), "memory",
+                                                    "dummy_extension_{0}".format(i))
+                CGroupsTelemetry.track_cgroup(dummy_memory_cgroup)
 
         self.assertEqual(2 * num_extensions, len(CGroupsTelemetry._tracked))
 
         with patch("azurelinuxagent.common.cgroup.MemoryCgroup.get_max_memory_usage") as patch_get_memory_max_usage:
             with patch("azurelinuxagent.common.cgroup.MemoryCgroup.get_memory_usage") as patch_get_memory_usage:
-                with patch("azurelinuxagent.common.cgroup.CpuCgroup._get_cpu_percent") as patch_get_cpu_percent:
-                    with patch("azurelinuxagent.common.cgroup.CpuCgroup._update_cpu_data") as patch_update_cpu_data:
-                        with patch("azurelinuxagent.common.cgroup.CGroup.is_active") as patch_is_active:
-                            for i in range(num_polls):
-                                patch_is_active.return_value = True
-                                patch_get_cpu_percent.return_value = cpu_percent_values[i]
-                                patch_get_memory_usage.return_value = memory_usage_values[i]  # example 200 MB
-                                patch_get_memory_max_usage.return_value = max_memory_usage_values[i]  # example 450 MB
-                                CGroupsTelemetry.poll_all_tracked()
+                with patch("azurelinuxagent.common.cgroup.CpuCgroup.get_cpu_usage") as patch_get_cpu_usage:
+                    with patch("azurelinuxagent.common.cgroup.CGroup.is_active") as patch_is_active:
+                        for i in range(num_polls):
+                            patch_is_active.return_value = True
+                            patch_get_cpu_usage.return_value = cpu_percent_values[i]
+                            patch_get_memory_usage.return_value = memory_usage_values[i]  # example 200 MB
+                            patch_get_memory_max_usage.return_value = max_memory_usage_values[i]  # example 450 MB
+                            CGroupsTelemetry.poll_all_tracked()
 
         performance_metrics = CGroupsTelemetry.report_all_tracked()
 
